﻿using System;
using System.IO;
using System.Windows.Forms;
using System.Collections.Generic;

namespace OrganizerPreview
{

    public partial class Form1 : Form
    {
        // Pfad und Dateiname der Kalenderdatei
        private string filename;

        FileInfo fileinfo;

        // Zeitspanne von 10 Tagen
        int days;
        private TimeSpan timespan;

        // Interval für Timer
        private int timerInterval;

        // Liest Kalenderdatei und erstellt Liste
        // mit Termin-Objekten
        private OrganizerReader or;

        // Enthält aktuelles Datum für die Auswahl
        // der anzuzeigenden Termine
        private DateTime actualDate;

        // Liste für die Termine in der Vorschau
        List<PreviewEvent> previewEvents;

        public Form1()
        {
            InitializeComponent();

            // Liste für Vorschautermine erstellen
            previewEvents = new List<PreviewEvent>();

            readSettings();

            timespan = new TimeSpan(days, 0, 0, 0);

            // Intervall für Timer

            timer.Interval = timerInterval;
            toolStripTextBoxInterval.Text = (timerInterval / 1000).ToString();
            toolStripStatusLabelInterval.Text = "Refresh Interval=" + (timerInterval / 1000).ToString() + "s";


            // Voreinstellung für TextBox im Menü
            toolStripTextBoxTimespan.Text = timespan.Days.ToString();

            // Titelzeile der Form anpassen
            this.Text = "OrganizerPreview [" + timespan.Days.ToString() + " Days]";


            if (filename != "")
            {
                // Nur Dateiname im Statuslabel anzeigen
                fileinfo = new FileInfo(filename);
                toolStripStatusLabelFilename.Text = fileinfo.Name;

                // Lesen der Datei und Erstellen von Ereignis-Objekten
                try
                {
                    or = new OrganizerReader(filename, timespan);
                    displayEvents();

                    timer.Enabled = true;
                }
                catch (IOException ex)
                {
                    timer.Enabled = false;
                    MessageBox.Show(ex.Message, "Error", MessageBoxButtons.OK, MessageBoxIcon.Error);
                }
            }
            else
            {
                listBoxOrganizerData.Items.Add("Inactive! Select valid Calendarfile.");
                toolStripStatusLabelFilename.Text = "No Calendarfile selected!";
            }
        }

        private void readSettings()
        {
            // Gesicherte Einstellungen lesen
            // Anzahl der Tage im voraus für die Terminanzeige
            days = Properties.OrganizerPreview.Default.Timespan;
            // Aktualisierungsintervall für Anzeige
            timerInterval = Properties.OrganizerPreview.Default.Interval * 1000;
            // Pfad und Dateiname der zu verwendenden Kalenderdatei
            filename = Properties.OrganizerPreview.Default.Filename;
            // Startposition und Größe Fenster wieder herstellen
            this.Location = Properties.OrganizerPreview.Default.WindowPosition;
            this.Size = Properties.OrganizerPreview.Default.WindowSize;
        }

        private void writeSettings()
        {
            // Einstellungen speichern
            Properties.OrganizerPreview.Default.Filename = filename;
            Properties.OrganizerPreview.Default.Timespan = days;
            Properties.OrganizerPreview.Default.Interval = timerInterval / 1000;
            Properties.OrganizerPreview.Default.WindowPosition = this.Location;
            Properties.OrganizerPreview.Default.WindowSize = this.Size;
            Properties.OrganizerPreview.Default.Save();

        }

        private void displayEvents()
        {
            actualDate = DateTime.Now;

            // Einträge in Vorschauliste löschen
            previewEvents.Clear();

            // Alle Ereignis-Objekte durchgehen und nur die Termine
            // innerhalb der Zeitspanne in Liste aufnehmen
            foreach (OrganizerEvent e in or.organizerEvents)
            {
                // Wenn der Termin innerhalb der nächste Tage
                // gemäß Zeitspanne liegt, in die Liste aufnehmen
                if ((e.DateStart.Date >= actualDate.Date) &&
                    (e.DateStart.Date <= actualDate.Date + timespan)
                   )

                {
                    // Vorschautermin erstellen und in Liste einfügen
                    PreviewEvent pe = new PreviewEvent(e.DateStart, e.Description);
                    previewEvents.Add(pe);
                }
                // Falls es ein Geburtstagseintrag ist,
                // nochmals gesondert testen
                else if (e.IsBirthday)
                {
                    // Jahreszahl für Geburtstageintrag neu setzen
                    DateTime birthday = new DateTime(actualDate.Year,
                                                     e.DateStart.Month,
                                                     e.DateStart.Day);

                    if ((birthday.Date >= actualDate.Date) &&
                        (birthday.Date <= actualDate.Date + timespan)
                       )
                    {
                        // Vorschautermin erstellen und in Liste einfügen
                        PreviewEvent pe = new PreviewEvent(birthday.Date, e.Description);
                        previewEvents.Add(pe);
                    }
                }
            }

            // Vorschauliste nach Datum sortieren
            previewEvents.Sort();

            // Listboxinhalt löschen
            listBoxOrganizerData.Items.Clear();
            // Listbox befüllen
            foreach (PreviewEvent item in previewEvents)
            {
                string ev = item.Date.ToShortDateString() + ": " + item.Description;
                listBoxOrganizerData.Items.Add(ev);
            }

        }

        private void toolStripTextBoxTimespan_KeyUp(object sender, KeyEventArgs e)
        {
            // Neue Zeitspanne in Tagen im Voraus in Menü-TextBox angeben
            if (e.KeyCode == Keys.Return)
            {
                try
                {
                    days = Convert.ToInt32(toolStripTextBoxTimespan.Text);
                }
                catch (Exception)
                {
                    MessageBox.Show("Wrong number!");
                    days = 10;
                    toolStripTextBoxTimespan.Text = days.ToString();
                }

                // Zeitspanne neu setzen und in Titelzeile korrigieren
                timespan = new TimeSpan(days, 0, 0, 0);
                this.Text = "OrganizerPreview [" + timespan.Days.ToString() + " Days]";

                // Menü-TextBox wieder schließen
                var item = (ToolStripItem)sender;
                var owner = (ToolStripDropDown)(item.Owner);
                owner.Close();

                // Anzeige aktualisieren für die neue Zeitspanne
                displayEvents();
            }
        }

        private void calenderfileToolStripMenuItem_Click(object sender, EventArgs e)
        {
            // Auswahl der zu verwendenden Kalenderdatei
            using (OpenFileDialog openFileDialog = new OpenFileDialog())
            {
                if (fileinfo != null)
                    // Öffnen im Verzeichnis der letzten Datei
                    openFileDialog.InitialDirectory = fileinfo.DirectoryName;
                else
                    openFileDialog.InitialDirectory = Environment.GetFolderPath(Environment.SpecialFolder.ApplicationData);

                openFileDialog.Filter = "All files (*.*)|*.*";

                if (openFileDialog.ShowDialog() == DialogResult.OK)
                {
                    filename = openFileDialog.FileName;
                    try
                    {
                        or = new OrganizerReader(filename, timespan);
                        displayEvents();

                        FileInfo fi = new FileInfo(filename);
                        toolStripStatusLabelFilename.Text = fi.Name;

                        timer.Enabled = true;
                    }
                    catch (IOException ex)
                    {
                        MessageBox.Show(ex.Message, "Error", MessageBoxButtons.OK, MessageBoxIcon.Error);
                        timer.Enabled = false;
                    }
                }
            }
        }

        private void Form1_FormClosing(object sender, FormClosingEventArgs e)
        {
            writeSettings();
        }

        private void Form1_Resize(object sender, EventArgs e)
        {
            // Größe der Listbox an Fenster anpassen
            listBoxOrganizerData.Width = this.Width - 45;
            listBoxOrganizerData.Height = this.Height - 100;
        }

        private void timer_Tick(object sender, EventArgs e)
        {
            // Kalenderdatei neu verarbeiten
            or = new OrganizerReader(filename, timespan);
            // Termine anzeigen
            displayEvents();
        }

        private void toolStripTextBoxInterval_KeyUp(object sender, KeyEventArgs e)
        {
            if (e.KeyCode == Keys.Return)
            {
                int interval = 0;
                try
                {
                    interval = Convert.ToInt32(toolStripTextBoxInterval.Text) * 1000;
                }
                catch (Exception)
                {
                    MessageBox.Show("Invalid Value", "Error", MessageBoxButtons.OK, MessageBoxIcon.Error);
                    toolStripTextBoxInterval.Text = (timerInterval / 1000).ToString();
                    return;
                }

                // Neuen Wert übernehmen und in Timer verwenden
                timerInterval = interval;
                timer.Interval = timerInterval;
                toolStripTextBoxInterval.Text = (timerInterval / 1000).ToString();
                toolStripStatusLabelInterval.Text = "Refresh Interval=" + (timerInterval / 1000).ToString() + "s";

                // Menü-TextBox wieder schließen
                var item = (ToolStripItem)sender;
                var owner = (ToolStripDropDown)(item.Owner);
                owner.Close();

            }
        }
    }
}
