﻿using System;
using System.Collections.Generic;

namespace OrganizerPreview
{
    // Kapselt die Termine aus der Kalenderdatei
    // Nur rudimentär, soweit benötigt
    public class OrganizerEvent : IComparable
    {
        // Startdatum eines Termins (DTSTART)
        public DateTime DateStart { private set; get; }

        // Beschreibung des Termins (SUMMARY)
        public string Description { private set; get; }

        public bool IsBirthday { private set; get; }

        // String-Liste mit den Textzeilen eines Termins
        // aus der Kalenderdatei; von OrganizerReader
        // per Konstruktor
        public List<string> EventText { private set; get; }

        public OrganizerEvent(List<string> eventtext)
        {
            EventText = eventtext;

            IsBirthday = false;

            // Iterieren der String-Liste
            foreach (string item in EventText)
            {
                // Zeile mit DTSTART auswerten als Starttermin
                // un diesen im Property DateStart eintragen
                if (item.StartsWith("DTSTART"))
                {
                    int position = item.IndexOf(":");
                    string date = item.Substring(position + 1, 8);
                    int year = Convert.ToInt32(date.Substring(0, 4));
                    int month = Convert.ToInt32(date.Substring(4, 2));
                    int day = Convert.ToInt32(date.Substring(6, 2));
                    DateStart = new DateTime(year, month, day);
                }
                // Zeile mit SUMMARY auswerten und im
                // Property Description eintragen.
                // Nur der Text hinter dem ersten Doppelpunkt!
                // Zeitweise ist hinter SUMMARY noch ein Zeichensatz
                // angegeben (SUMMARY;CHARSET=UTF-8:)
                else if (item.StartsWith("SUMMARY"))
                {
                    int position = item.IndexOf(":");
                    Description = item.Substring(position + 1);
                }
                // Für Geburtstage als jährliche Wiederholungstermine
                else if (item.StartsWith("RRULE"))
                {
                    int position = item.IndexOf(":");
                    if (item.Substring(position + 1, 3).ToUpper() == "YM1")
                        IsBirthday = true;
                }
            }
        }
        // Interface IComparable
        // um die Termine sortieren zu können anhand Datum
        public int CompareTo(object obj)
        {
            int rc = 0;
            OrganizerEvent oe = (OrganizerEvent)obj;

            if (this.DateStart < oe.DateStart) rc = -1;
            if (this.DateStart > oe.DateStart) rc = 1;
            return rc;
        }
    }
}
