﻿using System;
using System.Collections.Generic;

using System.IO;

namespace OrganizerPreview
{
    public class OrganizerReader
    {
        // Zu lesende Kalenderdatei
        public string Filename { private set; get; }

        // Zeitspanne, für die die Termine im Voraus
        // angezeigt werden sollen
        public TimeSpan Timespan { private set; get; }

        // Liste mit Termin-Objekten
        public List<OrganizerEvent> organizerEvents = new List<OrganizerEvent>();

        public OrganizerReader(string filename, TimeSpan timespan)
        {
            Filename = filename;
            Timespan = timespan;

            try
            {
                readOrganizerData();
            }
            catch (IOException ex)
            {
                throw ex;
            }

            // Sortieren nach Datum per IComparable
            organizerEvents.Sort();
        }

        // Lesen der Kalenderdatei und Erstellen der Termin-Objekte
        private void readOrganizerData()
        {
            using (StreamReader sw = new StreamReader(Filename))
            {
                if (!sw.EndOfStream)
                {
                    // Erste Zeile sollte BEGIN:VCALENDAR sein!
                    string line = sw.ReadLine();
                    if (line.ToUpper() != "BEGIN:VCALENDAR")
                    {
                        throw new IOException("Fileformat seems not to be a calenderfile");
                    }
                }

                // Lesen bis Dateiende, Zeile für Zeile
                while (!sw.EndOfStream)
                {
                    string line = sw.ReadLine();

                    // Termine beginnen mit BEGIN:VEVENT
                    if (line.ToUpper() == "BEGIN:VEVENT")
                    {
                        // String-Liste erstellen für Termin-Objekt
                        // und die Zeilen bis END:VEVENT darin speichern
                        List<string> eventdescription = new List<string>();
                        eventdescription.Add(line);

                        while((line = sw.ReadLine())!="END:VEVENT")
                        {
                            eventdescription.Add(line);
                        }
                        eventdescription.Add(line);

                        // Termin-Objekt erstekllen und die Liste mitliefern
                        OrganizerEvent oe = new OrganizerEvent(eventdescription);

                        // Termin-Objekt in Liste aller Termin-Objekte einhängen
                        organizerEvents.Add(oe);
                    }
                }
            }
        }
    }
}
